@@ -7,10 +7,12 @@ from django.core.exceptions import ImproperlyConfigured, ObjectDoesNotExist
 from django.db.models import Model
 
 from .models import AppliedMigration
+from .utils import itersubclasses
 
 import inspect
 import sys
 import inspect
+import re
 
 def is_a(klass=None, search_attr=None, fk=False, m2m=False, o2o=False,
                 exclude=False, delimiter=';', skip_missing=False):
@@ -44,83 +46,113 @@ def is_a(klass=None, search_attr=None, fk=False, m2m=False, o2o=False,
 class Migration(object):
     """Baseclass for each data migration"""
 
-    abstract = False
+    #: if set to True, this migration will be skipped
     skip = False
 
-    # model class the migration creates instances for
+    #: model class the migration creates instances for
     model = None
 
-    # SQL SELECT query which returns the data suitable for the new model
-    # structure
+    #: SQL SELECT query which returns the data suitable for the new model
+    #: structure
     query = None
 
-    # this should be a dict which describes the data returned by the query. You
-    # can supply the Class that the data should be
+    #: This has to be a dict which describes the data returned by the query.
+    #:
+    #: You can supply the Class that the data should be
     column_description = {}
 
-    # a list of classes that the model requires to be migrated before
-    # The User class for example is included into many other models
+    #: a list of classes that the model requires to be migrated before
+    #: The User class for example is included into many other models
     depends_on = []
 
-    # If the following is set to False, the migration will be executed only once
-    # Otherwise it will create missing elements
+    #: If the following is set to False, the migration will be executed only once
+    #: Otherwise it will create missing elements
     allow_updates = False
 
-    # this is a unique model field, which is used to search for existing
-    # model instances
-    #
-    # Example: for Django`s User model it is `username`
-    #
-    # REQUIRED if `allow_updates` is set to True
+    #: this is a unique model field, which is used to search for existing
+    #: model instances
+    #:
+    #: Example: for Django`s User model it can be `username` or `id`
+    #:
+    #: :important: this attribute is required if `allow_updates` is set to True
     search_attr = None
 
     #########
     # Hooks #
     #########
-
-    # This function will be called for each row the query returns
-    # after the model has been instantiated or saved.
-    #
-    # Parameters:
-    #   1. the current model instance
-    #   2. The current row returned from the query
     @classmethod
     def hook_before_transformation(self, row):
+        """Is called right before row is passed to the model constructor.
+
+        Manipulate the row data if it is required. Here you can bring the data
+        in a suitable form which is not possible in SQL itself.
+
+        :param row: the dict which represents one row of the SQL query
+        """
         pass
 
+
     @classmethod
     def hook_before_save(self, instance, row):
+        """Is called right before the migrated instance is saved.
+
+        Do the changes, that make the instance valid, in this hook.
+
+        :param instance: the migrating instance which could be altered
+        :param row: the dict which represents one row of the SQL query
+        """
         pass
 
+
     @classmethod
     def hook_after_save(self, instance, row):
+        """Is called right after the migrated instance has been saved initially.
+
+        This is the place where you can set the data for a DateTimeField with
+        `auto_now_add`, where the date from the SQL query is not used
+        otherwise.
+
+        :param instance: the migrating instance which could be altered
+        :param row: the dict which represents one row of the SQL query
+        :note: when you make changes to the instance you have to call `save()`
+               manually
+        """
         pass
 
+
     @classmethod
     def hook_update_existing(self, instance, row):
-        """
-        Is called on the existing instance when `self.allow_updates` is True
+        """Is called for each existing instance when `allow_updates` is True
 
-        `row` contains the raw result without any transformation
+        :param instance: the existing instance which can be updated
+        :param row: contains the raw result without any transformation
 
-        Is is YOUR responsibility to make sure, that this method can be called
-        MULTIPLE times. DO SOME CHECKS
+        :note: It is YOUR responsibility to make sure, that this method can be
+               called MULTIPLE times. DO SOME CHECKS
         """
         pass
 
 
-    # hooks that will be called [before|after] all
-    # data [has been|will be] migrated
-    #
-    # BUT NOT FOR MIGRATIONS THAT WILL BE UPDATED
     @classmethod
     def hook_before_all(self):
+        """Is called before the migration will be migrated
+
+        Here you can execute some special setup code, which should be executed
+        only once.
+        """
         pass
 
+
     @classmethod
     def hook_after_all(self):
+        """Is called after the migration has been processed succesfully
+
+        Here you can set a debugger breakpoint for testing the migration
+        results.
+        """
         pass
 
+
     ###################
     # INTERNAL THINGS #
     ###################
@@ -156,7 +188,7 @@ class Migration(object):
     @classmethod
     def open_db_connection(self):
         raise ImproperlyConfigured(
-            "You have to supply a suitable db connection for your DB")
+            "You have to supply a suitable db connection for your DB: %s" % self)
 
 
     @classmethod
@@ -197,7 +229,8 @@ class Migration(object):
         for row in cursor.fetchall():
 
             # search for an existing instance
-            desc = is_a(self.model, search_attr=self.search_attr, skip_missing=True)
+            desc = is_a(self.model, search_attr=self.search_attr,
+                        fk=True, skip_missing=True)
             element = self.get_object(desc, row[self.search_attr])
 
             if element is not None:
@@ -254,15 +287,15 @@ class Migration(object):
                     if data is None:
                         continue
 
-                    usernames = data.split(desc['delimiter'])
-                    users = []
+                    parts = data.split(desc['delimiter'])
+                    objects = []
 
-                    for name in usernames:
-                        element = self.get_object(desc, name)
+                    for part in parts:
+                        element = self.get_object(desc, part)
                         if element is None:
                             continue
-                        users.append(element)
-                    m2ms[fieldname] = users
+                        objects.append(element)
+                    m2ms[fieldname] = objects
 
             else:
                 constructor_data[fieldname] = data
@@ -322,9 +355,10 @@ class Migration(object):
             raise ImproperlyConfigured(
                     '%s: `model` has to be a model CLASS' % self)
 
-        if not ( isinstance(self.query, str) and "SELECT" in self.query ):
+        if not re.search('SELECT', self.query, re.IGNORECASE|re.MULTILINE):
             raise ImproperlyConfigured(
-                    '%s: `model` has to be a string containing SELECT' % self)
+                '%s: `query` has to be a string containing SELECT: %s' % (
+                    self, self.query))
 
 
 from django.conf import settings
@@ -342,7 +376,7 @@ class Importer(object):
     @classmethod
     def possible_existing_migrations(self):
         return [ app + ".data_migration_spec"
-            for app in  self.installed_apps() ]
+            for app in self.installed_apps() ]
 
     @classmethod
     def import_all(self, excludes=[]):
@@ -406,11 +440,12 @@ class Migrator(object):
 
     @classmethod
     def sorted_migrations(self):
-        return self.__sort_based_on_dependency(Migration.__subclasses__())
+        return self.sort_based_on_dependency(
+            [ mig for mig in itersubclasses(Migration) if mig.query ])
 
 
     @classmethod
-    def __sort_based_on_dependency(self, classes):
+    def sort_based_on_dependency(self, classes):
         """sorts the given Migration classes based on their dependencies
             to other Models.
 
